/*
 * Example for TouchKey 2 click
 *
 * Date         Feb 2017
 * Author       Stefan Popovic
 * Copyright    GNU Public License v2
 *
 * Test configuration STM32 :
 *  MCU           :	STM32F107VC
 *  Dev. Board    :	EasyMx PRO v7 for STM32 ARM
 *  SW            :	mikroC PRO for ARM v4.9.0
 *
 *  NOTE          : - Place TouchKey 2 click board on mikroBUS slot 1
 *                  - Connect easyTFT and turn backlight ON ( turn SW11.3 ) 
 *                  - This click board sends 3 bytes via UART, on key pressed  
 *                   ( 0xAA |  KEY_Byte | 0xFE )
 *
/*******************************************************************************
* Includes
*******************************************************************************/
#include "tft_resources.h"
#include <stdbool.h>
#include <stdint.h>

/*******************************************************************************
* Module Preprocessor Constants
*******************************************************************************/
#define BUFFER_SIZE             3
#define KEY_A                0x01
#define KEY_B                0x02
#define KEY_C                0x04
#define KEY_D                0x08

/*******************************************************************************
* Module Variable Definitions
*******************************************************************************/
sbit TOUCH_KEY_2_RST at GPIOC_ODR.B2;
sbit TOUCH_KEY_2_CS  at GPIOD_ODR.B13;

bool key_A_pressed  = false;
bool key_B_pressed  = false;
bool key_C_pressed  = false;
bool key_D_pressed  = false;
bool no_key_pressed = true;

char buffer[ BUFFER_SIZE ] = { 0 };

uint16_t ind = 0;
uint8_t old_key = 0;

/*******************************************************************************
* Function Declarations
*******************************************************************************/
static void system_init( void );
static void check_key_event( void );
static void display_key( char* key_name, unsigned int color );

/*******************************************************************************
* Function Definitions
*******************************************************************************/

/*
 * System Initialization
 *
 * Configuring MCU ports, initializing UART, enabling interrupt
 * and performing reset
 */
static void system_init( void )
{   
    // Configuring RST and CS pins as output
    GPIO_Digital_Output( &GPIOC_ODR, _GPIO_PINMASK_2 );
    GPIO_Digital_Output( &GPIOD_ODR, _GPIO_PINMASK_13 );

    // UART Initialization
    UART3_Init_Advanced( 9600, _UART_8_BIT_DATA,
                                _UART_NOPARITY, 
                                _UART_ONE_STOPBIT, 
                                &_GPIO_MODULE_USART3_PD89 );
    Delay_ms( 100 );

    // UART Interrupt enabling
    RXNEIE_USART3_CR1_bit = 1;
    NVIC_IntEnable( IVT_INT_USART3 );
    EnableInterrupts();

    // Perform TouchKey 2 board reset 
    TOUCH_KEY_2_RST = 1;
    Delay_ms( 100 );
    TOUCH_KEY_2_RST = 0;                                
    Delay_ms( 100 );
    TOUCH_KEY_2_RST = 1;
}

/*
 * Checking for key pressed events
 *
 * Looking for Key ID and setting appropriate flags. Called inside interrupt
 */
static void check_key_event ( void )
{
        if ( buffer[ ind ] == 0xAA &&
             buffer[ (ind + 1) % BUFFER_SIZE ] == KEY_A &&
             buffer[ (ind + 2) % BUFFER_SIZE ] == 0xFE )
             { 
                if ( no_key_pressed )
                     no_key_pressed = false; 
                else  old_key = KEY_A;
                key_A_pressed = true;
             }
        else if ( buffer[ ind ] == 0xAA &&
                  buffer[ (ind + 1) % BUFFER_SIZE ] == KEY_B &&
                  buffer[ (ind + 2) % BUFFER_SIZE ] == 0xFE )
             {
                if ( no_key_pressed )
                     no_key_pressed = false; 
                else  old_key = KEY_B; 
                key_B_pressed = true;
             }
         else if ( buffer[ ind ] == 0xAA &&
                   buffer[ (ind + 1) % BUFFER_SIZE ] == KEY_C &&
                   buffer[ (ind + 2) % BUFFER_SIZE ] == 0xFE )
             {
                if ( no_key_pressed )
                     no_key_pressed = false; 
                else   old_key = KEY_C;
                key_C_pressed = true;
             }
         else if ( buffer[ ind ] == 0xAA &&
                   buffer[ (ind + 1) % BUFFER_SIZE ] == KEY_D &&
                   buffer[ (ind + 2) % BUFFER_SIZE ] == 0xFE )
            {
                if ( no_key_pressed )
                     no_key_pressed = false; 
                else   old_key = KEY_D;
                key_D_pressed = true;
            }   
          
         ++ind;
         if ( ind == BUFFER_SIZE )  ind = 0;
         
}

/*
 * Displaying key name with color
 */
void display_key( unsigned char* key_name, unsigned int color )
{
  TFT_Set_Font( &LilyUPC57x120_Bold, color, FO_HORIZONTAL );
  TFT_Rectangle( 50, 80, 300, 165 );
  TFT_Write_Text( key_name, 135, 80 );
}

/*
 * Main function
 */
void main( void )
{
    system_init();
    display_init();

    TFT_Set_Font( &HandelGothic_BT21x22_Regular, CL_RED, FO_HORIZONTAL );
    TFT_Write_Text( "Press Any On-board Key", 50, 120 );

    while( true ) 
    {
    
        // Key A has been pressed
        if( key_A_pressed && old_key != KEY_A )
        {   
            display_key( "A", CL_RED );
            key_A_pressed = false;
            old_key = KEY_A;
        }
        // Key B has been pressed
        else if( key_B_pressed && old_key != KEY_B )
        {
            display_key( "B", CL_GREEN );
            key_B_pressed = false;
            old_key = KEY_B;
        }
        // Key C has been pressed
        else if( key_C_pressed && old_key != KEY_C )
        {
            display_key( "C", CL_BLUE );
            key_C_pressed = false;
            old_key = KEY_C;
        }
        // Key D has been pressed
        else if( key_D_pressed && old_key != KEY_D )
        {
            display_key( "D", CL_PURPLE );
            key_D_pressed = false;
            old_key = KEY_D;
        }
    }
}

void Touch_Key_2_RX_ISR() iv IVT_INT_USART3 ics ICS_AUTO
{
    if( RXNE_USART3_SR_bit )
    {
        char tmp = USART3_DR;
        buffer[ ind ] = tmp;
        check_key_event();
    }
}
/************************************************************ END OF FILE *****/